﻿#ifndef CGAME_H
#define CGAME_H

#include <QImage>
#include "../common/sources/system/cmaths.h"
#include "../common/sources/Map.h"
#include "character.h"
#include "mouse.h"
#include "sound.h"

#define SCREEN_WIDTH    320
#define SCREEN_HEIGHT   200
#define MAINVIEW_WIDTH  224
#define MAINVIEW_HEIGHT 136

#define ORNATE_MIRROR           6
#define ORNATE_TORCH_FULL       7
#define ORNATE_TORCH_EMPTY      8
#define ORNATE_ARCHED_ALCOVE    9
#define ORNATE_TEXT             12
#define ORNATE_IRON_LOCK        13
#define ORNATE_LEVER_OFF        32

#define MIN_LIGHT_LEVEL         0.05f
#define MAX_LIGHT_LEVEL         1.0f

class CSnapshot
{
public:
    CSnapshot();
    CSnapshot(const CSnapshot& rhs);
    virtual ~CSnapshot();

    CSnapshot&  operator=(const CSnapshot& rhs);
    void    copyPressPlates(bool* src);
    void    load(FILE* handle);
    void    save(FILE* handle);

    int     level;
    CMap    map;
    bool*   pressPlateStates;

private:
    void    freePressPlates();
};

//---------------------------------------------------------------------------------------------
class CGame
{
public:
    CGame();

    QImage& mainLoop();
    void    loadLevel(int level, CVec2 pos, int side);
    void    snapshotLevel();
    void    saveGame(const char* fileName);
    void    loadGame(const char *fileName);

    CCharacter  characters[4];
    CWall*      lastMirrorClicked;
    int         currentLevel;
    int         frameCounter;

private:
    void    drawWall(QImage* image, CVec2 mapPos, CVec2 tablePos, EWallSide side, bool flip);
    void    displayMainView(QImage* image);
    void    update(SMouseArea* clickedArea);
    float   getLightLevel();
    void    teleportStacks(CVec2 mapPos);
    void    fallStacks(CVec2 mapPos);

    std::vector<CSnapshot>   mVisitedLevels;
};

extern CGame    game;

extern CSoundManager   *sound;

#endif // CGAME_H
